@@ -59,6 +59,17 @@ def test_abort():
     """
     abort("Test")
 
+class TestException(Exception):
+    pass
+
+@raises(TestException)
+def test_abort_with_exception():
+    """
+    abort() should raise a provided exception
+    """
+    with settings(abort_exception=TestException):
+        abort("Test")
+
 
 @mock_streams('stderr')
 @with_patched_object(output, 'aborts', True)
@@ -84,6 +95,27 @@ def test_puts_with_user_output_on():
     puts(s, show_prefix=False)
     eq_(sys.stdout.getvalue(), s + "\n")
 
+@mock_streams('stdout')
+def test_puts_with_unicode_output():
+    """
+    puts() should print unicode input
+    """
+    s = u"string!"
+    output.user = True
+    puts(s, show_prefix=False)
+    eq_(sys.stdout.getvalue(), s + "\n")
+
+
+@mock_streams('stdout')
+def test_puts_with_encoding_type_none_output():
+    """
+    puts() should print unicode output without a stream encoding
+    """
+    s = u"string!"
+    output.user = True
+    sys.stdout.encoding = None
+    puts(s, show_prefix=False)
+    eq_(sys.stdout.getvalue(), s + "\n")
 
 @mock_streams('stdout')
 def test_puts_with_user_output_off():
